import Delta from "quill-delta-es";
import { type BlockyElement, type AttributesObject, type BlockyNode, type BlockElement } from "./tree";
import { type Operation, type OperationMessage } from "./operations";
import { NodeLocation } from "./location";
import type { CursorState } from "./cursor";
export declare enum ChangesetRecordOption {
    Undo = 0,
    Redo = 1,
    None = 2
}
export interface ChangesetApplyOptions {
    updateView: boolean;
    ignoreCursor: boolean;
    record: ChangesetRecordOption;
    refreshCursor: boolean;
}
export interface ChangesetStateLogger {
    userId: string;
    get appliedVersion(): number;
    get cursorState(): CursorState | null;
    getLocationOfNode(node: BlockyNode): NodeLocation;
    apply(changeset: FinalizedChangeset): void;
}
/**
 * Changeset is a collection of changes which can be
 * applied to the document's state.
 *
 * Each changeset has it's own ID.
 * So it can be applied repeatedly.
 */
export declare class Changeset {
    readonly state: ChangesetStateLogger;
    version: number;
    operations: Operation[];
    beforeCursor: CursorState | null;
    afterCursor?: CursorState | null;
    forceUpdate: boolean;
    constructor(state: ChangesetStateLogger);
    /**
     * Update the attributes of a `BlockyNode`.
     */
    updateAttributes(node: BlockyElement, attributes: AttributesObject): Changeset;
    /**
     * Set the cursor after the changeset is applied.
     */
    setCursorState(cursorState: CursorState | null): Changeset;
    /**
     * Append a node at the end of another node.
     */
    appendChild(node: BlockyElement, child: BlockyNode): Changeset;
    removeChild(parent: BlockyElement, child: BlockyNode): Changeset;
    /**
     * Remove a node from the parent.
     */
    removeNode(node: BlockyNode): Changeset;
    /**
     * Delete a sequences of children of a node.
     */
    deleteChildrenAt(parent: BlockyElement, index: number, count: number): Changeset;
    /**
     * Insert a sequences of children after another node.
     */
    insertChildrenAfter(parent: BlockyElement, children: BlockyNode[], after?: BlockyNode | null): Changeset;
    /**
     * Insert children at the position of a node.
     */
    insertChildrenAt(parent: BlockyElement, index: number, children: BlockyNode[]): Changeset;
    /**
     * Apply the quill delta to the BlockyNode.
     * For a `Text` block, the `propName` is usually called `textContent`.
     */
    textEdit(node: BlockElement, propName: string, delta: () => Delta): Changeset;
    textConcat(node: BlockElement, propName: string, delta: () => Delta): Changeset;
    /**
     * This method will transform path.
     * If you don't want to transform path, call [pushWillMerge].
     */
    push(operation: Operation): void;
    pushWillMerge(operation: Operation): void;
    /**
     * Apply this changeset to the `EditorState`.
     * The editor will render automatically after the changeset is applied.
     */
    apply(options?: Partial<ChangesetApplyOptions>): FinalizedChangeset | void;
    finalize(options?: Partial<ChangesetApplyOptions>): FinalizedChangeset;
    /**
     * @param that Can be either a [Changeset] or a [FinalizedChangeset]
     */
    append(that: Changeset | FinalizedChangeset): void;
}
export interface FinalizedChangeset {
    userId: string;
    version: number;
    operations: Operation[];
    beforeCursor: CursorState | null;
    afterCursor?: CursorState | null;
    forceUpdate: boolean;
    options: ChangesetApplyOptions;
}
/**
 * A changeset interface used to transmit.
 * Can be serialized as JSON.
 */
export interface ChangesetMessage {
    userId: string;
    version: number;
    operations: OperationMessage[];
    beforeCursor: CursorState | null;
    afterCursor?: CursorState | null;
    forceUpdate: boolean;
    options: ChangesetApplyOptions;
}
/**
 * Convert a {@link FinalizedChangeset} to {@link ChangesetMessage}.
 */
export declare function changesetToMessage(changeset: FinalizedChangeset): ChangesetMessage;
/**
 * Convert a {@link ChangesetMessage} to {@link FinalizedChangeset}.
 */
export declare function changesetFromMessage(msg: ChangesetMessage): FinalizedChangeset;
